@@ -89,6 +89,17 @@ class BehanceGalleryExtractor(BehanceExtractor):
         BehanceExtractor.__init__(self, match)
         self.gallery_id = match.group(1)
 
+    def _init(self):
+        BehanceExtractor._init(self)
+
+        modules = self.config("modules")
+        if modules:
+            if isinstance(modules, str):
+                modules = modules.split(",")
+            self.modules = set(modules)
+        else:
+            self.modules = {"image", "video", "mediacollection", "embed"}
+
     def items(self):
         data = self.get_gallery_data()
         imgs = self.get_images(data)
@@ -97,7 +108,8 @@ class BehanceGalleryExtractor(BehanceExtractor):
         yield Message.Directory, data
         for data["num"], (url, module) in enumerate(imgs, 1):
             data["module"] = module
-            data["extension"] = text.ext_from_url(url)
+            data["extension"] = (module.get("extension") or
+                                 text.ext_from_url(url))
             yield Message.Url, url, data
 
     def get_gallery_data(self):
@@ -133,13 +145,25 @@ class BehanceGalleryExtractor(BehanceExtractor):
         append = result.append
 
         for module in data["modules"]:
-            mtype = module["__typename"]
+            mtype = module["__typename"][:-6].lower()
 
-            if mtype == "ImageModule":
-                url = module["imageSizes"]["size_original"]["url"]
-                append((url, module))
+            if mtype not in self.modules:
+                self.log.debug("Skipping '%s' module", mtype)
+                continue
 
-            elif mtype == "VideoModule":
+            if mtype == "image":
+                sizes = {
+                    size["url"].rsplit("/", 2)[1]: size
+                    for size in module["imageSizes"]["allAvailable"]
+                }
+                size = (sizes.get("source") or
+                        sizes.get("max_3840") or
+                        sizes.get("fs") or
+                        sizes.get("hd") or
+                        sizes.get("disp"))
+                append((size["url"], module))
+
+            elif mtype == "video":
                 try:
                     renditions = module["videoData"]["renditions"]
                 except Exception:
@@ -158,7 +182,7 @@ class BehanceGalleryExtractor(BehanceExtractor):
 
                 append((url, module))
 
-            elif mtype == "MediaCollectionModule":
+            elif mtype == "mediacollection":
                 for component in module["components"]:
                     for size in component["imageSizes"].values():
                         if size:
@@ -167,12 +191,17 @@ class BehanceGalleryExtractor(BehanceExtractor):
                             append(("/".join(parts), module))
                             break
 
-            elif mtype == "EmbedModule":
+            elif mtype == "embed":
                 embed = module.get("originalEmbed") or module.get("fluidEmbed")
                 if embed:
                     embed = text.unescape(text.extr(embed, 'src="', '"'))
+                    module["extension"] = "mp4"
                     append(("ytdl:" + embed, module))
 
+            elif mtype == "text":
+                module["extension"] = "txt"
+                append(("text:" + module["text"], module))
+
         return result
 
 
