@@ -13,7 +13,7 @@ from .. import text
 
 BASE_PATTERN = (
     r"(?:https?://)?(?:app\.)?(bunkr+"
-    r"\.(?:s[kiu]|ru|la|is|to|ac|black|cat|media|red|site|ws))"
+    r"\.(?:s[kiu]|fi|ru|la|is|to|ac|black|cat|media|red|site|ws))"
 )
 
 LEGACY_DOMAINS = {
@@ -54,7 +54,6 @@ class BunkrAlbumExtractor(LolisafeAlbumExtractor):
             "album_id"   : self.album_id,
             "album_name" : text.unescape(info[0]),
             "album_size" : size[1:-1],
-            "description": text.unescape(info[2]) if len(info) > 2 else "",
             "count"      : len(urls),
         }
 
