@@ -165,9 +165,10 @@ class InstagramExtractor(Extractor):
             data = {
                 "post_id" : post["pk"],
                 "post_shortcode": post["code"],
-                "likes": post["like_count"],
+                "likes": post.get("like_count", 0),
                 "pinned": post.get("timeline_pinned_user_ids", ()),
                 "date": text.parse_timestamp(post.get("taken_at")),
+                "liked": post.get("has_liked", False),
             }
 
             caption = post["caption"]
@@ -270,6 +271,7 @@ class InstagramExtractor(Extractor):
             "typename"   : typename,
             "date"       : text.parse_timestamp(post["taken_at_timestamp"]),
             "likes"      : post["edge_media_preview_like"]["count"],
+            "liked"      : post.get("viewer_has_liked", False),
             "pinned"     : pinned,
             "owner_id"   : owner["id"],
             "username"   : owner.get("username"),
@@ -594,6 +596,22 @@ class InstagramTagExtractor(InstagramExtractor):
         return self.api.tags_media(self.item)
 
 
+class InstagramInfoExtractor(InstagramExtractor):
+    """Extractor for an Instagram user's profile data"""
+    subcategory = "info"
+    pattern = USER_PATTERN + r"/info"
+    example = "https://www.instagram.com/USER/info/"
+
+    def items(self):
+        screen_name = self.item
+        if screen_name.startswith("id:"):
+            user = self.api.user_by_id(screen_name[3:])
+        else:
+            user = self.api.user_by_name(screen_name)
+
+        return iter(((Message.Directory, user),))
+
+
 class InstagramAvatarExtractor(InstagramExtractor):
     """Extractor for an Instagram user's avatar"""
     subcategory = "avatar"
@@ -689,7 +707,10 @@ class InstagramRestAPI():
     def reels_media(self, reel_ids):
         endpoint = "/v1/feed/reels_media/"
         params = {"reel_ids": reel_ids}
-        return self._call(endpoint, params=params)["reels_media"]
+        try:
+            return self._call(endpoint, params=params)["reels_media"]
+        except KeyError:
+            raise exception.AuthorizationError("Login required")
 
     def tags_media(self, tag):
         for section in self.tags_sections(tag):
@@ -733,7 +754,7 @@ class InstagramRestAPI():
                 not user["followed_by_viewer"]:
             name = user["username"]
             s = "" if name.endswith("s") else "s"
-            raise exception.StopExtraction("%s'%s posts are private", name, s)
+            self.extractor.log.warning("%s'%s posts are private", name, s)
         self.extractor._assign_user(user)
         return user["id"]
 
@@ -970,14 +991,14 @@ class InstagramGraphqlAPI():
             if not info["has_next_page"]:
                 return extr._update_cursor(None)
             elif not data["edges"]:
-                s = "" if self.item.endswith("s") else "s"
+                s = "" if self.extractor.item.endswith("s") else "s"
                 raise exception.StopExtraction(
-                    "%s'%s posts are private", self.item, s)
+                    "%s'%s posts are private", self.extractor.item, s)
 
             variables["after"] = extr._update_cursor(info["end_cursor"])
 
 
-@cache(maxage=90*24*3600, keyarg=1)
+@cache(maxage=90*86400, keyarg=1)
 def _login_impl(extr, username, password):
     extr.log.error("Login with username & password is no longer supported. "
                    "Use browser cookies instead.")
