@@ -237,11 +237,214 @@ def parse_command_line(module, argv):
             getattr(opts, "sponsorblock_mark", None) or set()
         opts.sponsorblock_remove = \
             getattr(opts, "sponsorblock_remove", None) or set()
-    sponsorblock_query = opts.sponsorblock_mark | opts.sponsorblock_remove
     opts.remove_chapters = getattr(opts, "remove_chapters", None) or ()
 
-    # PostProcessors
+    try:
+        postprocessors = list(module.get_postprocessors(opts))
+    except AttributeError:
+        postprocessors = legacy_postprocessors(
+            opts, module, ytdlp, compat_opts)
+
+    match_filter = (
+        None if opts.match_filter is None
+        else module.match_filter_func(opts.match_filter))
+
+    cookiesfrombrowser = getattr(opts, "cookiesfrombrowser", None)
+    if cookiesfrombrowser:
+        match = re.fullmatch(r"""(?x)
+            (?P<name>[^+:]+)
+            (?:\s*\+\s*(?P<keyring>[^:]+))?
+            (?:\s*:\s*(?!:)(?P<profile>.+?))?
+            (?:\s*::\s*(?P<container>.+))?
+        """, cookiesfrombrowser)
+        if match:
+            browser, keyring, profile, container = match.groups()
+            if keyring is not None:
+                keyring = keyring.upper()
+            cookiesfrombrowser = (browser.lower(), profile, keyring, container)
+        else:
+            cookiesfrombrowser = None
+
+    return {
+        "usenetrc": opts.usenetrc,
+        "netrc_location": getattr(opts, "netrc_location", None),
+        "username": opts.username,
+        "password": opts.password,
+        "twofactor": opts.twofactor,
+        "videopassword": opts.videopassword,
+        "ap_mso": opts.ap_mso,
+        "ap_username": opts.ap_username,
+        "ap_password": opts.ap_password,
+        "quiet": opts.quiet,
+        "no_warnings": opts.no_warnings,
+        "forceurl": opts.geturl,
+        "forcetitle": opts.gettitle,
+        "forceid": opts.getid,
+        "forcethumbnail": opts.getthumbnail,
+        "forcedescription": opts.getdescription,
+        "forceduration": opts.getduration,
+        "forcefilename": opts.getfilename,
+        "forceformat": opts.getformat,
+        "forceprint": getattr(opts, "forceprint", None) or (),
+        "force_write_download_archive": getattr(
+            opts, "force_write_download_archive", None),
+        "simulate": opts.simulate,
+        "skip_download": opts.skip_download,
+        "format": opts.format,
+        "allow_unplayable_formats": getattr(
+            opts, "allow_unplayable_formats", None),
+        "ignore_no_formats_error": getattr(
+            opts, "ignore_no_formats_error", None),
+        "format_sort": getattr(
+            opts, "format_sort", None),
+        "format_sort_force": getattr(
+            opts, "format_sort_force", None),
+        "allow_multiple_video_streams": opts.allow_multiple_video_streams,
+        "allow_multiple_audio_streams": opts.allow_multiple_audio_streams,
+        "check_formats": getattr(
+            opts, "check_formats", None),
+        "outtmpl": opts.outtmpl,
+        "outtmpl_na_placeholder": opts.outtmpl_na_placeholder,
+        "paths": getattr(opts, "paths", None),
+        "autonumber_size": opts.autonumber_size,
+        "autonumber_start": opts.autonumber_start,
+        "restrictfilenames": opts.restrictfilenames,
+        "windowsfilenames": getattr(opts, "windowsfilenames", None),
+        "ignoreerrors": opts.ignoreerrors,
+        "force_generic_extractor": opts.force_generic_extractor,
+        "ratelimit": opts.ratelimit,
+        "throttledratelimit": getattr(opts, "throttledratelimit", None),
+        "overwrites": getattr(opts, "overwrites", None),
+        "retries": opts.retries,
+        "file_access_retries": getattr(opts, "file_access_retries", None),
+        "fragment_retries": opts.fragment_retries,
+        "extractor_retries": getattr(opts, "extractor_retries", None),
+        "skip_unavailable_fragments": opts.skip_unavailable_fragments,
+        "keep_fragments": opts.keep_fragments,
+        "concurrent_fragment_downloads": getattr(
+            opts, "concurrent_fragment_downloads", None),
+        "buffersize": opts.buffersize,
+        "noresizebuffer": opts.noresizebuffer,
+        "http_chunk_size": opts.http_chunk_size,
+        "continuedl": opts.continue_dl,
+        "noprogress": True if opts.noprogress is None else opts.noprogress,
+        "playliststart": opts.playliststart,
+        "playlistend": opts.playlistend,
+        "playlistreverse": opts.playlist_reverse,
+        "playlistrandom": opts.playlist_random,
+        "noplaylist": opts.noplaylist,
+        "logtostderr": outtmpl_default == "-",
+        "consoletitle": opts.consoletitle,
+        "nopart": opts.nopart,
+        "updatetime": opts.updatetime,
+        "writedescription": opts.writedescription,
+        "writeannotations": opts.writeannotations,
+        "writeinfojson": opts.writeinfojson,
+        "allow_playlist_files": opts.allow_playlist_files,
+        "clean_infojson": opts.clean_infojson,
+        "getcomments": getattr(opts, "getcomments", None),
+        "writethumbnail": opts.writethumbnail is True,
+        "write_all_thumbnails": getattr(opts, "write_all_thumbnails", None) or
+        opts.writethumbnail == "all",
+        "writelink": getattr(opts, "writelink", None),
+        "writeurllink": getattr(opts, "writeurllink", None),
+        "writewebloclink": getattr(opts, "writewebloclink", None),
+        "writedesktoplink": getattr(opts, "writedesktoplink", None),
+        "writesubtitles": opts.writesubtitles,
+        "writeautomaticsub": opts.writeautomaticsub,
+        "allsubtitles": opts.allsubtitles,
+        "subtitlesformat": opts.subtitlesformat,
+        "subtitleslangs": opts.subtitleslangs,
+        "matchtitle": module.decodeOption(opts.matchtitle),
+        "rejecttitle": module.decodeOption(opts.rejecttitle),
+        "max_downloads": opts.max_downloads,
+        "prefer_free_formats": opts.prefer_free_formats,
+        "trim_file_name": getattr(opts, "trim_file_name", None),
+        "verbose": opts.verbose,
+        "dump_intermediate_pages": opts.dump_intermediate_pages,
+        "write_pages": opts.write_pages,
+        "test": opts.test,
+        "keepvideo": opts.keepvideo,
+        "min_filesize": opts.min_filesize,
+        "max_filesize": opts.max_filesize,
+        "min_views": opts.min_views,
+        "max_views": opts.max_views,
+        "daterange": date,
+        "cachedir": opts.cachedir,
+        "youtube_print_sig_code": opts.youtube_print_sig_code,
+        "age_limit": opts.age_limit,
+        "download_archive": download_archive_fn,
+        "break_on_existing": getattr(opts, "break_on_existing", None),
+        "break_on_reject": getattr(opts, "break_on_reject", None),
+        "break_per_url": getattr(opts, "break_per_url", None),
+        "skip_playlist_after_errors": getattr(
+            opts, "skip_playlist_after_errors", None),
+        "cookiefile": opts.cookiefile,
+        "cookiesfrombrowser": cookiesfrombrowser,
+        "nocheckcertificate": opts.no_check_certificate,
+        "prefer_insecure": opts.prefer_insecure,
+        "proxy": opts.proxy,
+        "socket_timeout": opts.socket_timeout,
+        "bidi_workaround": opts.bidi_workaround,
+        "debug_printtraffic": opts.debug_printtraffic,
+        "prefer_ffmpeg": opts.prefer_ffmpeg,
+        "include_ads": opts.include_ads,
+        "default_search": opts.default_search,
+        "dynamic_mpd": getattr(opts, "dynamic_mpd", None),
+        "extractor_args": getattr(opts, "extractor_args", None),
+        "youtube_include_dash_manifest": getattr(
+            opts, "youtube_include_dash_manifest", None),
+        "youtube_include_hls_manifest": getattr(
+            opts, "youtube_include_hls_manifest", None),
+        "encoding": opts.encoding,
+        "extract_flat": opts.extract_flat,
+        "live_from_start": getattr(opts, "live_from_start", None),
+        "wait_for_video": getattr(opts, "wait_for_video", None),
+        "mark_watched": opts.mark_watched,
+        "merge_output_format": opts.merge_output_format,
+        "postprocessors": postprocessors,
+        "fixup": opts.fixup,
+        "source_address": opts.source_address,
+        "sleep_interval_requests": getattr(
+            opts, "sleep_interval_requests", None),
+        "sleep_interval": opts.sleep_interval,
+        "max_sleep_interval": opts.max_sleep_interval,
+        "sleep_interval_subtitles": getattr(
+            opts, "sleep_interval_subtitles", None),
+        "external_downloader": opts.external_downloader,
+        "playlist_items": opts.playlist_items,
+        "xattr_set_filesize": opts.xattr_set_filesize,
+        "match_filter": match_filter,
+        "no_color": getattr(opts, "no_color", None),
+        "ffmpeg_location": opts.ffmpeg_location,
+        "hls_prefer_native": opts.hls_prefer_native,
+        "hls_use_mpegts": opts.hls_use_mpegts,
+        "hls_split_discontinuity": getattr(
+            opts, "hls_split_discontinuity", None),
+        "external_downloader_args": opts.external_downloader_args,
+        "postprocessor_args": opts.postprocessor_args,
+        "cn_verification_proxy": opts.cn_verification_proxy,
+        "geo_verification_proxy": opts.geo_verification_proxy,
+        "geo_bypass": getattr(
+            opts, "geo_bypass", "default"),
+        "geo_bypass_country": getattr(
+            opts, "geo_bypass_country", None),
+        "geo_bypass_ip_block": getattr(
+            opts, "geo_bypass_ip_block", None),
+        "compat_opts": compat_opts,
+    }
+
+
+def parse_retries(retries, name=""):
+    if retries in ("inf", "infinite"):
+        return float("inf")
+    return int(retries)
+
+
+def legacy_postprocessors(opts, module, ytdlp, compat_opts):
     postprocessors = []
+
+    sponsorblock_query = opts.sponsorblock_mark | opts.sponsorblock_remove
     if opts.metafromtitle:
         postprocessors.append({
             "key": "MetadataFromTitle",
@@ -371,178 +574,4 @@ def parse_command_line(module, argv):
             "when": "after_move",
         })
 
-    match_filter = (
-        None if opts.match_filter is None
-        else module.match_filter_func(opts.match_filter))
-
-    return {
-        "usenetrc": opts.usenetrc,
-        "netrc_location": getattr(opts, "netrc_location", None),
-        "username": opts.username,
-        "password": opts.password,
-        "twofactor": opts.twofactor,
-        "videopassword": opts.videopassword,
-        "ap_mso": opts.ap_mso,
-        "ap_username": opts.ap_username,
-        "ap_password": opts.ap_password,
-        "quiet": opts.quiet,
-        "no_warnings": opts.no_warnings,
-        "forceurl": opts.geturl,
-        "forcetitle": opts.gettitle,
-        "forceid": opts.getid,
-        "forcethumbnail": opts.getthumbnail,
-        "forcedescription": opts.getdescription,
-        "forceduration": opts.getduration,
-        "forcefilename": opts.getfilename,
-        "forceformat": opts.getformat,
-        "forceprint": getattr(opts, "forceprint", None) or (),
-        "force_write_download_archive": getattr(
-            opts, "force_write_download_archive", None),
-        "simulate": opts.simulate,
-        "skip_download": opts.skip_download,
-        "format": opts.format,
-        "allow_unplayable_formats": getattr(
-            opts, "allow_unplayable_formats", None),
-        "ignore_no_formats_error": getattr(
-            opts, "ignore_no_formats_error", None),
-        "format_sort": getattr(
-            opts, "format_sort", None),
-        "format_sort_force": getattr(
-            opts, "format_sort_force", None),
-        "allow_multiple_video_streams": opts.allow_multiple_video_streams,
-        "allow_multiple_audio_streams": opts.allow_multiple_audio_streams,
-        "check_formats": getattr(
-            opts, "check_formats", None),
-        "outtmpl": opts.outtmpl,
-        "outtmpl_na_placeholder": opts.outtmpl_na_placeholder,
-        "paths": getattr(opts, "paths", None),
-        "autonumber_size": opts.autonumber_size,
-        "autonumber_start": opts.autonumber_start,
-        "restrictfilenames": opts.restrictfilenames,
-        "windowsfilenames": getattr(opts, "windowsfilenames", None),
-        "ignoreerrors": opts.ignoreerrors,
-        "force_generic_extractor": opts.force_generic_extractor,
-        "ratelimit": opts.ratelimit,
-        "throttledratelimit": getattr(opts, "throttledratelimit", None),
-        "overwrites": getattr(opts, "overwrites", None),
-        "retries": opts.retries,
-        "file_access_retries": getattr(opts, "file_access_retries", None),
-        "fragment_retries": opts.fragment_retries,
-        "extractor_retries": getattr(opts, "extractor_retries", None),
-        "skip_unavailable_fragments": opts.skip_unavailable_fragments,
-        "keep_fragments": opts.keep_fragments,
-        "concurrent_fragment_downloads": getattr(
-            opts, "concurrent_fragment_downloads", None),
-        "buffersize": opts.buffersize,
-        "noresizebuffer": opts.noresizebuffer,
-        "http_chunk_size": opts.http_chunk_size,
-        "continuedl": opts.continue_dl,
-        "noprogress": True if opts.noprogress is None else opts.noprogress,
-        "playliststart": opts.playliststart,
-        "playlistend": opts.playlistend,
-        "playlistreverse": opts.playlist_reverse,
-        "playlistrandom": opts.playlist_random,
-        "noplaylist": opts.noplaylist,
-        "logtostderr": outtmpl_default == "-",
-        "consoletitle": opts.consoletitle,
-        "nopart": opts.nopart,
-        "updatetime": opts.updatetime,
-        "writedescription": opts.writedescription,
-        "writeannotations": opts.writeannotations,
-        "writeinfojson": opts.writeinfojson,
-        "allow_playlist_files": opts.allow_playlist_files,
-        "clean_infojson": opts.clean_infojson,
-        "getcomments": getattr(opts, "getcomments", None),
-        "writethumbnail": opts.writethumbnail is True,
-        "write_all_thumbnails": getattr(opts, "write_all_thumbnails", None) or
-        opts.writethumbnail == "all",
-        "writelink": getattr(opts, "writelink", None),
-        "writeurllink": getattr(opts, "writeurllink", None),
-        "writewebloclink": getattr(opts, "writewebloclink", None),
-        "writedesktoplink": getattr(opts, "writedesktoplink", None),
-        "writesubtitles": opts.writesubtitles,
-        "writeautomaticsub": opts.writeautomaticsub,
-        "allsubtitles": opts.allsubtitles,
-        "subtitlesformat": opts.subtitlesformat,
-        "subtitleslangs": opts.subtitleslangs,
-        "matchtitle": module.decodeOption(opts.matchtitle),
-        "rejecttitle": module.decodeOption(opts.rejecttitle),
-        "max_downloads": opts.max_downloads,
-        "prefer_free_formats": opts.prefer_free_formats,
-        "trim_file_name": getattr(opts, "trim_file_name", None),
-        "verbose": opts.verbose,
-        "dump_intermediate_pages": opts.dump_intermediate_pages,
-        "write_pages": opts.write_pages,
-        "test": opts.test,
-        "keepvideo": opts.keepvideo,
-        "min_filesize": opts.min_filesize,
-        "max_filesize": opts.max_filesize,
-        "min_views": opts.min_views,
-        "max_views": opts.max_views,
-        "daterange": date,
-        "cachedir": opts.cachedir,
-        "youtube_print_sig_code": opts.youtube_print_sig_code,
-        "age_limit": opts.age_limit,
-        "download_archive": download_archive_fn,
-        "break_on_existing": getattr(opts, "break_on_existing", None),
-        "break_on_reject": getattr(opts, "break_on_reject", None),
-        "break_per_url": getattr(opts, "break_per_url", None),
-        "skip_playlist_after_errors": getattr(
-            opts, "skip_playlist_after_errors", None),
-        "cookiefile": opts.cookiefile,
-        "cookiesfrombrowser": getattr(opts, "cookiesfrombrowser", None),
-        "nocheckcertificate": opts.no_check_certificate,
-        "prefer_insecure": opts.prefer_insecure,
-        "proxy": opts.proxy,
-        "socket_timeout": opts.socket_timeout,
-        "bidi_workaround": opts.bidi_workaround,
-        "debug_printtraffic": opts.debug_printtraffic,
-        "prefer_ffmpeg": opts.prefer_ffmpeg,
-        "include_ads": opts.include_ads,
-        "default_search": opts.default_search,
-        "dynamic_mpd": getattr(opts, "dynamic_mpd", None),
-        "extractor_args": getattr(opts, "extractor_args", None),
-        "youtube_include_dash_manifest": getattr(
-            opts, "youtube_include_dash_manifest", None),
-        "youtube_include_hls_manifest": getattr(
-            opts, "youtube_include_hls_manifest", None),
-        "encoding": opts.encoding,
-        "extract_flat": opts.extract_flat,
-        "live_from_start": getattr(opts, "live_from_start", None),
-        "wait_for_video": getattr(opts, "wait_for_video", None),
-        "mark_watched": opts.mark_watched,
-        "merge_output_format": opts.merge_output_format,
-        "postprocessors": postprocessors,
-        "fixup": opts.fixup,
-        "source_address": opts.source_address,
-        "sleep_interval_requests": getattr(
-            opts, "sleep_interval_requests", None),
-        "sleep_interval": opts.sleep_interval,
-        "max_sleep_interval": opts.max_sleep_interval,
-        "sleep_interval_subtitles": getattr(
-            opts, "sleep_interval_subtitles", None),
-        "external_downloader": opts.external_downloader,
-        "playlist_items": opts.playlist_items,
-        "xattr_set_filesize": opts.xattr_set_filesize,
-        "match_filter": match_filter,
-        "no_color": opts.no_color,
-        "ffmpeg_location": opts.ffmpeg_location,
-        "hls_prefer_native": opts.hls_prefer_native,
-        "hls_use_mpegts": opts.hls_use_mpegts,
-        "hls_split_discontinuity": getattr(
-            opts, "hls_split_discontinuity", None),
-        "external_downloader_args": opts.external_downloader_args,
-        "postprocessor_args": opts.postprocessor_args,
-        "cn_verification_proxy": opts.cn_verification_proxy,
-        "geo_verification_proxy": opts.geo_verification_proxy,
-        "geo_bypass": opts.geo_bypass,
-        "geo_bypass_country": opts.geo_bypass_country,
-        "geo_bypass_ip_block": opts.geo_bypass_ip_block,
-        "compat_opts": compat_opts,
-    }
-
-
-def parse_retries(retries, name=""):
-    if retries in ("inf", "infinite"):
-        return float("inf")
-    return int(retries)
+    return postprocessors
