@@ -30,7 +30,8 @@ class MockPostprocessorModule(Mock):
 
 class FakeJob():
 
-    def __init__(self, extr=extractor.find("test:")):
+    def __init__(self, extr=extractor.find("generic:https://example.org/")):
+        extr.directory_fmt = ("{category}",)
         self.extractor = extr
         self.pathfmt = path.PathFormat(extr)
         self.out = output.NullOutput()
@@ -171,7 +172,7 @@ class ExecTest(BasePostprocessorTest):
             "command": "echo {} {_path} {_directory} {_filename} && rm {};",
         })
 
-        with patch("subprocess.Popen") as p:
+        with patch("gallery_dl.util.Popen") as p:
             i = Mock()
             i.wait.return_value = 0
             p.return_value = i
@@ -191,7 +192,7 @@ class ExecTest(BasePostprocessorTest):
                         "\fE _directory.upper()"],
         })
 
-        with patch("subprocess.Popen") as p:
+        with patch("gallery_dl.util.Popen") as p:
             i = Mock()
             i.wait.return_value = 0
             p.return_value = i
@@ -211,7 +212,7 @@ class ExecTest(BasePostprocessorTest):
             "command": "echo {}",
         })
 
-        with patch("subprocess.Popen") as p:
+        with patch("gallery_dl.util.Popen") as p:
             i = Mock()
             i.wait.return_value = 123
             p.return_value = i
@@ -229,7 +230,7 @@ class ExecTest(BasePostprocessorTest):
             "command": "echo {}",
         })
 
-        with patch("subprocess.Popen") as p:
+        with patch("gallery_dl.util.Popen") as p:
             i = Mock()
             p.return_value = i
             self._trigger(("after",))
@@ -439,6 +440,18 @@ class MetadataTest(BasePostprocessorTest):
         path = self.pathfmt.realdirectory + "metadata/file.json"
         m.assert_called_once_with(path, "w", encoding="utf-8")
 
+    def test_metadata_directory_format(self):
+        self._create(
+            {"directory": ["..", "json", "\fE str(id // 500 * 500 + 500)"]},
+            {"id": 12345},
+        )
+
+        with patch("builtins.open", mock_open()) as m:
+            self._trigger()
+
+        path = self.pathfmt.realdirectory + "../json/12500/file.ext.json"
+        m.assert_called_once_with(path, "w", encoding="utf-8")
+
     def test_metadata_filename(self):
         self._create({
             "filename"        : "{category}_{filename}_/meta/\n\r.data",
@@ -572,6 +585,16 @@ class MtimeTest(BasePostprocessorTest):
         self._trigger()
         self.assertEqual(self.pathfmt.kwdict["_mtime"], 315532800)
 
+    def test_mtime_none(self):
+        self._create(None, {"date": None})
+        self._trigger()
+        self.assertNotIn("_mtime", self.pathfmt.kwdict)
+
+    def test_mtime_undefined(self):
+        self._create(None, {})
+        self._trigger()
+        self.assertNotIn("_mtime", self.pathfmt.kwdict)
+
     def test_mtime_key(self):
         self._create({"key": "foo"}, {"foo": 315532800})
         self._trigger()
